-- VTOP.vhd
-- --------------------------------------------------------
-- Top-Level Example for XESS board FPGA, linked with 8031
-- --------------------------------------------------------
-- Bertrand Cuzeau
-- alse@compuserve.com
--
-- Design :
--  * F000 = LED register,  W/O
--  * E000 = LFSR register, R/W,
--  * D000 = LFSR shift command = W/O (data unimportant)
-- To be used with "WALK.asm"
--
-- Note :
--  This design is fully synchronous, and seems
--  to work just fine. However, it relies on the
--  8031 timing which I did not check (nor have).
--  A synchronous design has so many advantages
--  that it is worth finding ways to adhere to
--  this methodology.

LIBRARY ieee;
  USE ieee.std_logic_1164.ALL;
--USE ieee.numeric_std.ALL; don't need in this simple design


-- --------------------------------------------
    Entity VTOP is
-- --------------------------------------------
    Port (   CLK : In    std_logic;
           nPSEN : In    std_logic;
            PC_D : In    std_logic_vector (5 downto 0);
             nWR : In    std_logic;
             ALE : In    std_logic;
              AH : In   std_logic_vector (15 downto 8);
              AD : InOut std_logic_vector (7 downto 0);
              AL : Out  std_logic_vector  (7 downto 0);
             nCE : Out   std_logic;
             nRD : In    std_logic;
             nOE : Out   std_logic;
           AH_16 : Out   std_logic;
             RST : Out   std_logic;
           XTAL1 : Out   std_logic;
             LED : Out   std_logic_vector (6 downto 0)
            );
end VTOP;

-- --------------------------------------------
    Architecture SCHEMATIC of VTOP is
-- --------------------------------------------

  component G_DEC
     Port (       C : In    std_logic;
                CLK : In    std_logic;
                  D : In    std_logic;
                  E : In    std_logic;
                  Q : Out   std_logic );
  end component;

  component G_BIDIR
     Port (       I : In    std_logic;
                 OE : In    std_logic;
                  O : InOut std_logic;
                FBK : Out   std_logic );
  end component;

  component LFSREG
     Port (     CLK : In    std_logic;
                  D : In    std_logic_vector (7 downto 0);
                 EN : In    std_logic;
                 LD : In    std_logic;
                RST : In    std_logic;
                  Q : Out   std_logic_vector (7 downto 0) );
  end component;

  component LEDREG
     Port (     CLK : In    std_logic;
                  D : In    std_logic_vector (7 downto 0);
                RST : In    std_logic;
                 EN : In    std_logic;
                  Q : Out   std_logic_vector (7 downto 0) );
  end component;
-- --------------------------------------------

SIGNAL      GND : std_logic; -- May be useful in port maps ('87 style).
SIGNAL      VCC : std_logic; -- May be useful in port maps ('87 style).

signal  LFShift : std_logic;
signal   LDLFSR : std_logic;
signal    ENLED : std_logic;
signal      DOE : std_logic;
signal     RSTi : std_logic;
signal      Din : std_logic_vector (7 downto 0);
signal     Dout : std_logic_vector (7 downto 0);
signal     RLED : std_logic_vector (7 downto 0);
signal       WR : std_logic;
signal       RD : std_logic;

-- --------------------------------------------
begin

GND  <= '0';      -- initialize here, not at declaration !
VCC  <= '1';      -- initialize here, not at declaration !

AH_16 <= '0';     -- Make sure we work on the lowest 64k Ram

RSTi <= PC_D(0);  -- D0, // port is our Global Reset

XTAL1 <= CLK;     -- pass the clock thru to the uC

nCE <= AH(15);
nOE <= not (not nRD or not nPSEN);

WR   <= not nWR;
RD   <= not nRD;

RST  <= RSTi;

process (AH,RD,WR)
begin
  LDLFSR  <= '0';
  LFShift <= '0';
  DOE     <= '0';
  ENLED   <= '0';
  if    (AH(15 downto 12)= x"F") then
    -- DOE <= RD;   -- if register were readable...
    ENLED <= WR;
  elsif (AH(15 downto 12)= x"E") then
    LDLFSR <= WR;
    DOE <= RD;      -- we can read back this register
  elsif (AH(15 downto 12)= x"D") then
    LFShift <= WR;
  end if;
end process;

LFSR1 : LFSREG Port Map (
          CLK=>CLK, 
          RST=>RSTi,
          D=>Din(7 downto 0), 
          EN=>LFShift,
          LD=>LDLFSR, 
          Q=>Dout(7 downto 0) );


LEDR1 : LEDREG  Port Map (
          CLK=>CLK,
          RST=>RSTi, 
          D=>Din(7 downto 0), 
          EN=>ENLED,
          Q=>RLED(7 downto 0) );

LED <= RLED (6 downto 0); -- MSB is ignored

-- Latch AD into A[7:0] with Clock, Enabled by ALE
-- (generate is for the fun : the process below is simpler)

--GENR1: for i in 0 to 7 generate
--  DL1: G_DEC Port Map ( C=>RSTi, CLK=>CLK, E=>ALE, D=>Din(i), Q=>AL(i) );
--end generate;

process (CLK,RSTi)
begin
  if RSTi='1' then
    AL <= (others=>'0');
  elsif rising_edge(CLK) then
    if ALE='1' then
      AL <= Din;
    end if;
  end if;
end process;


-- Bidirectional Bus AD --
-- same remark as above : the process is simpler

--BID1: for i in 0 to 7 generate
-- BI1: G_BIDIR Port Map ( I=>Dout(i), OE=>DOE, O=>AD(i), FBK=>Din(i) );
--end generate;

Din <= AD;  -- FeedBack

process (AD,Dout,DOE)  -- TriState output
begin
  if DOE='1' then
    AD <= Dout;
  else
    AD <= (others=>'Z');
  end if;
end process;

end SCHEMATIC;

